/* eslint-disable max-len */
const gulp = require('gulp');
const maps = require('gulp-sourcemaps');
const cleanCSS = require('gulp-clean-css');
const sass = require('gulp-sass');
const concat = require('gulp-concat');
const uglify = require('gulp-uglify-es').default;
const filter = require('gulp-filter');
const rename = require('gulp-rename');
const ngAnnotate = require('gulp-ng-annotate');
const browserSync = require('browser-sync').create();

const { reload } = browserSync;
const del = require('del');
const removeEmptyLines = require('gulp-remove-empty-lines');
const replace = require('gulp-string-replace');
const babel = require('gulp-babel');
const historyApiFallback = require('connect-history-api-fallback');

const src = {
  scss: ['app/scss/*.scss', 'app/scss/views/*.scss'],
  css: 'app/build/css',
  html: 'app/**/*.html',
  js: ['avatar-module/**', 'app/app.js', 'app/js/**/*.js', 'app/js/*.js'],
};

// Static Server + watching scss/html files
gulp.task('serve', () => {
  browserSync.init({
    server: {
      baseDir: ['./app', './avatar-module'],
      middleware: [historyApiFallback()],
    },
    open: false,
    reloadDelay: 250,
    https: false,
  });
  gulp.watch(src.scss, gulp.series('sass'));
  gulp.watch(src.html).on('change', reload);
  gulp.watch(src.js, gulp.series('appjs')).on('change', reload);
});

gulp.task('vendorjs', () => {
  const filterJS = filter('**/*.js', { restore: true });

  return gulp.src([
    'node_modules/angular/angular.js',
    'node_modules/angular-ui-router/release/angular-ui-router.js',
    'node_modules/angular-sanitize/angular-sanitize.js',
    'node_modules/angular-file-saver/dist/angular-file-saver.bundle.min.js',
  ])
    .pipe(filterJS)
    .pipe(maps.init())
    .pipe(concat('vendor.min.js'))
    .pipe(uglify())
    .pipe(maps.write('./'))
    .pipe(gulp.dest('app/build/js/'));
});

gulp.task('vendorcss', () => gulp.src([
  'node_modules/components-font-awesome/css/fontawesome.css',
])
  .pipe(maps.init())
  .pipe(concat('vendor.min.css'))
  .pipe(cleanCSS())
  .pipe(maps.write('./'))
  .pipe(gulp.dest('app/build/css/')));

gulp.task('appjs', () => gulp.src([
  'avatar-module/**',
  'app/app.js',
  'app/js/**/*.js',
])
  .pipe(replace('const cacheVersion = 1', `const cacheVersion = ${Date.now()}`))
  .pipe(babel({
    presets: ['@babel/preset-env'],
    plugins: ['@babel/plugin-proposal-object-rest-spread'],
  }))
  .pipe(concat('app.min.js'))
  .pipe(ngAnnotate())
  .pipe(gulp.dest('app/build/js/')));

gulp.task('appjs:minify', () => gulp.src([
  'avatar-module/**',
  'app/app.js',
  'app/js/**/*.js',
])
  .pipe(maps.init())
  .pipe(replace('const cacheVersion = 1', `const cacheVersion = ${Date.now()}`))
  .pipe(babel({
    presets: ['@babel/preset-env'],
    plugins: ['@babel/plugin-proposal-object-rest-spread'],
  }))
  .pipe(concat('app.min.js'))
  .pipe(ngAnnotate())
  .pipe(uglify())
  .pipe(maps.write('./'))
  .pipe(gulp.dest('app/build/js/')));

// Compile sass into CSS
gulp.task('sass', () => gulp.src(src.scss)
  .pipe(sass())
  .pipe(gulp.dest(src.css))
  .pipe(reload({ stream: true })));

gulp.task('build-index', () => gulp.src([
  'app/app.html',
])
  .pipe(replace('vendor.min.css', `vendor.min.css?v=${Date.now()}`))
  .pipe(replace('vendor.min.js', `vendor.min.js?v=${Date.now()}`))
  .pipe(replace('app.min.js', `app.min.js?v=${Date.now()}`))
  .pipe(rename('index.html'))
  .pipe(gulp.dest('app/'))
  .pipe(removeEmptyLines())
  .pipe(gulp.dest('app/')));

gulp.task('clean', () => del([
  './public/**/*',
]));

gulp.task('copy-index', () => gulp.src([
  'app/index.html',
  'app/favicon.png',
  'app/.htaccess',
  'app/robots.txt',
])
  .pipe(gulp.dest('public')));

gulp.task('copy-assets', () => gulp.src([
  'app/assets/**/*',
])
  .pipe(gulp.dest('public/assets')));

gulp.task('copy-build', () => gulp.src([
  'app/build/**/*',
])
  .pipe(gulp.dest('public/build')));

gulp.task('copy-templates', () => gulp.src([
  'app/templates/**/*',
])
  .pipe(gulp.dest('public/templates')));

gulp.task('copy-languages', () => gulp.src([
  'app/languages/**/*',
])
  .pipe(gulp.dest('public/languages')));

gulp.task('bundle-min', () => gulp.src([
  'avatar-module/avatar-module.js',
  'avatar-module/js/**/*.js',
])
  .pipe(replace('const cacheVersion = 1', `const cacheVersion = ${Date.now()}`))
  .pipe(babel({
    presets: ['@babel/preset-env'],
    plugins: ['@babel/plugin-proposal-object-rest-spread'],
  }))
  .pipe(concat('avatar.min.js'))
  .pipe(ngAnnotate())
  .pipe(uglify())
  .pipe(gulp.dest('dist/')));

gulp.task('bundle-concat', () => gulp.src([
  'avatar-module/avatar-module.js',
  'avatar-module/js/**/*.js',
])
  .pipe(replace('const cacheVersion = 1', `const cacheVersion = ${Date.now()}`))
  .pipe(babel({
    presets: ['@babel/preset-env'],
    plugins: ['@babel/plugin-proposal-object-rest-spread'],
  }))
  .pipe(concat('avatar.js'))
  .pipe(ngAnnotate())
  .pipe(gulp.dest('dist/')));

exports.bundle = gulp.parallel('bundle-concat', 'bundle-min');
exports.build = gulp.parallel('vendorjs', exports.bundle, 'vendorcss', 'appjs', 'sass', 'build-index');
exports.minifyBuild = gulp.parallel('vendorjs', exports.bundle, 'vendorcss', 'appjs:minify', 'sass', 'build-index');
exports.copy = gulp.series('copy-index', 'copy-assets', 'copy-build', 'copy-templates', 'copy-languages');
exports.buildRelease = gulp.series('clean', exports.minifyBuild, exports.copy);
exports.default = gulp.series(exports.build, 'serve');
